/*****************************************************************************
 *   rgb.c:  Driver for the RGB LED
 *
 *   Copyright(C) 2009, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "mcu_regs.h"
#include "type.h"
#include "gpio.h"
#include "rgb.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize RGB driver
 *
 *****************************************************************************/
void rgb_init (void)
{
    GPIOSetDir( PORT0, 21, 1 );
    GPIOSetDir( PORT0, 14, 1 );
    GPIOSetDir( PORT0, 22, 1 );

    LPC_IOCON->TRST_PIO0_14 &= ~0x0F;
    LPC_IOCON->TRST_PIO0_14 |= 0x01;
}


/******************************************************************************
 *
 * Description:
 *    Set LED states
 *
 * Params:
 *    [in]  ledMask  - The mask is used to turn LEDs on or off
 *
 *****************************************************************************/
void rgb_setLeds (uint8_t ledMask)
{
    if ((ledMask & RGB_RED) != 0) {
    	GPIOSetBitValue( PORT0, 21, 1 );
    } else {
    	GPIOSetBitValue( PORT0, 21, 0 );
    }

    if ((ledMask & RGB_BLUE) != 0) {
    	GPIOSetBitValue( PORT0, 14, 1 );
    } else {
    	GPIOSetBitValue( PORT0, 14, 0 );
    }

    if ((ledMask & RGB_GREEN) != 0) {
    	GPIOSetBitValue( PORT0, 22, 1 );
    } else {
    	GPIOSetBitValue( PORT0, 22, 0 );
    }

}
