/*****************************************************************************
 *   rgb.c:  Driver for the RGB LED
 *
 *   Copyright(C) 2009, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "mcu_regs.h"
#include "type.h"
#include "gpio.h"
#include "rgb.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize RGB driver
 *
 *****************************************************************************/
void rgb_init (void)
{
    GPIOSetDir( PORT1, 9, 1 );
    GPIOSetDir( PORT1, 2, 1 );
    GPIOSetDir( PORT1, 10, 1 );

    LPC_IOCON->PIO1_10 = LPC_IOCON->PIO1_10 & ~0x7;
    LPC_IOCON->PIO1_9 = LPC_IOCON->PIO1_9 & ~0x7;
    LPC_IOCON->JTAG_nTRST_PIO1_2 = (LPC_IOCON->JTAG_nTRST_PIO1_2 & ~0x7) | 0x1;
}


/******************************************************************************
 *
 * Description:
 *    Set LED states
 *
 * Params:
 *    [in]  ledMask  - The mask is used to turn LEDs on or off
 *
 *****************************************************************************/
void rgb_setLeds (uint8_t ledMask)
{
    if ((ledMask & RGB_RED) != 0) {
        GPIOSetValue( PORT1, 9, 1 );
    } else {
        GPIOSetValue( PORT1, 9, 0 );
    }

    if ((ledMask & RGB_BLUE) != 0) {
        GPIOSetValue( PORT1, 2, 1 );
    } else {
        GPIOSetValue( PORT1, 2, 0 );
    }

    if ((ledMask & RGB_GREEN) != 0) {
        GPIOSetValue( PORT1, 10, 1 );
    } else {
        GPIOSetValue( PORT1, 10, 0 );
    }

}
