/*****************************************************************************
 *   rgb.c:  Driver for the RGB LED
 *
 *   Copyright(C) 2009, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "mcu_regs.h"
#include "lpc_types.h"
#include "gpio.h"
#include "rgb.h"
#include "sysctrl.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize RGB driver
 *
 *****************************************************************************/
void rgb_init (void)
{
  SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO1, ENABLE);
  SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO2, ENABLE);

  GPIO_SetDir(LPC_GPIO2, 0, 1);
  GPIO_SetDir(LPC_GPIO1, 5, 1);
  GPIO_SetDir(LPC_GPIO2, 4, 1);
}


/******************************************************************************
 *
 * Description:
 *    Set LED states
 *
 * Params:
 *    [in]  ledMask  - The mask is used to turn LEDs on or off
 *
 *****************************************************************************/
void rgb_setLeds (uint8_t ledMask)
{
    if ((ledMask & RGB_RED) != 0) {
        GPIO_SetHighLevel(LPC_GPIO2, 0, 1);
    } else {
        GPIO_SetLowLevel(LPC_GPIO2, 0, 1);
    }

    if ((ledMask & RGB_BLUE) != 0) {
        GPIO_SetHighLevel(LPC_GPIO1, 5, 1);
    } else {
        GPIO_SetLowLevel(LPC_GPIO1, 5, 1);
    }

    if ((ledMask & RGB_GREEN) != 0) {
        GPIO_SetHighLevel(LPC_GPIO2, 4, 1);
    } else {
        GPIO_SetLowLevel(LPC_GPIO2, 4, 1);
    }

}
