/*****************************************************************************
 *   This example is is playing a WAV file
 *
 *   Copyright(C) 2009, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/


#include "mcu_regs.h"
#include "lpc_types.h"
#include "board.h"
#include "uart.h"
#include "stdio.h"
#include "timer32.h"
#include "gpio.h"
#include "iocon.h"
#include "sysctrl.h"

extern const unsigned char sound_8k[];
extern int sound_sz;


int main (void) {

  int i = 0;
  uint32_t cnt = 0;
  uint32_t off = 0;
  uint32_t sampleRate = 0;
  IOCON_PIO_CFG_Type PIO_mode;

  brd_usb_uart_init();

  UART_SendString(DEV_USB_UART, (uint8_t*)"Speaker - Wav\r\n");

  SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO0, ENABLE);
  SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO1, ENABLE);
  SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO2, ENABLE);

  GPIO_SetDir(LPC_GPIO2, 0, 1);
  GPIO_SetDir(LPC_GPIO2, 4, 1);

  GPIO_SetDir(LPC_GPIO0, 28, 1);
  GPIO_SetDir(LPC_GPIO0, 29, 1);
  GPIO_SetDir(LPC_GPIO1, 6, 1);
  GPIO_SetDir(LPC_GPIO1, 5, 1);

  GPIO_SetLowLevel(LPC_GPIO0, 28, 0); //LM4811-clk
  GPIO_SetLowLevel(LPC_GPIO0, 29, 0); //LM4811-up/dn
  GPIO_SetLowLevel(LPC_GPIO1, 6, 0);  //LM4811-shutdn


  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<8);

  IOCON_StructInit(&PIO_mode);
  PIO_mode.type = IOCON_CT16_B1_MAT0_LOC1;
  IOCON_SetFunc(&PIO_mode);

  LPC_CT16B1->TCR = 2;     //reset counter
  LPC_CT16B1->TCR = 0;
  LPC_CT16B1->PR = 0;      //prescaler = 0
  LPC_CT16B1->MCR = (1<<10)|(1<<9); //reset TC on MC3 match
  LPC_CT16B1->MR3 = 286;   //set period     //256
  LPC_CT16B1->MR0 = 127;   //set duty cycle
  LPC_CT16B1->PWMC = 1;    //enable PWM on MAT0 output
  LPC_CT16B1->IR = 0xf;
  LPC_CT16B1->TCR = 1;     //counter enable


  /* ChunkID */
  if (sound_8k[cnt] != 'R' && sound_8k[cnt+1] != 'I' &&
      sound_8k[cnt+2] != 'F' && sound_8k[cnt+3] != 'F')
  {
    UART_SendString(DEV_USB_UART, (uint8_t*)"Wrong format (RIFF)\r\n");
    return 0;
  }
  cnt+=4;

  /* skip chunk size*/
  cnt += 4;

  /* Format */
  if (sound_8k[cnt] != 'W' && sound_8k[cnt+1] != 'A' &&
      sound_8k[cnt+2] != 'V' && sound_8k[cnt+3] != 'E')
  {
    UART_SendString(DEV_USB_UART, (uint8_t*)"Wrong format (WAVE)\r\n");
    return 0;
  }
  cnt+=4;

  /* SubChunk1ID */
  if (sound_8k[cnt] != 'f' && sound_8k[cnt+1] != 'm' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != ' ')
  {
    UART_SendString(DEV_USB_UART, (uint8_t*)"Missing fmt\r\n");
    return 0;
  }
  cnt+=4;

  /* skip chunk size, audio format, num channels */
  cnt+= 8;

  sampleRate = (sound_8k[cnt] | (sound_8k[cnt+1] << 8) |
      (sound_8k[cnt+2] << 16) | (sound_8k[cnt+3] << 24));

  if (sampleRate != 8000) {
    UART_SendString(DEV_USB_UART, (uint8_t*)"Only 8kHz supported\r\n");
    return 0;
  }

  cnt+=4;

  /* skip byte rate, align, bits per sample */
  cnt += 8;

  /* SubChunk2ID */
  if (sound_8k[cnt] != 'd' && sound_8k[cnt+1] != 'a' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != 'a')
  {
    UART_SendString(DEV_USB_UART, (uint8_t*)"Missing data\r\n");
    return 0;
  }
  cnt += 4;

  /* skip chunk size */
  cnt += 4;

  off = cnt;

  while(1)
  {
    int val;


    cnt = off;
    while(cnt++ < sound_sz)
    {

      LPC_CT16B1->MR0 = sound_8k[cnt]+30;

      for (val=0;val<12;val++)
      {
        while (!(LPC_CT16B1->IR & 0x8));
        LPC_CT16B1->IR = 0xf;
      }
    }

    //delay
    for(i=0; i<0x100000; i++)
      __ASM(" nop");
  }


  return 0 ;
}


